/**********************************************************************\
 * CellGrid
 * 		Handles low-level cellular automata world
\**********************************************************************/

package Life;

import java.util.Random;

public class CellGrid {
	private int length;
	private int height;
	private boolean array[][];	

	// create grid
	public CellGrid(int l, int h) {
		length = l;
		height = h;
		array = new boolean[l][h];
	}
	
	public int getLength() {
		return length;
	}
	
	public int getHeight() {
		return height;
	}
	
	// change cell value
	public void setCell(int i, int j, boolean val) {
		array[i][j] = val;
	}

	// return cell value (wrap-around)
	public boolean getCell(int i, int j) {
		i = (i + length) % (length);
		j = (j + height) % (height);
		return array[i][j];
	}
	
	// determine next state of cell	
	public boolean checkCell(int i, int j) {
		int n = 0;
		if (getCell(i - 1, j -1)) n = n + 1;
		if (getCell(i, j - 1)) n = n + 1;
		if (getCell(i + 1, j - 1)) n = n + 1;
		if (getCell(i - 1, j)) n = n + 1;
		if (getCell(i + 1, j)) n = n + 1;
		if (getCell(i - 1, j + 1)) n = n + 1;
		if (getCell(i, j + 1)) n = n + 1;
		if (getCell(i + 1, j + 1)) n = n + 1;
	
		if (getCell(i, j))
			return !(n <= 1 || n >= 4);
		else
			return (n == 3);
	} 

	// create next generation	
	public void makeArray() {
		boolean newArray[][] = new boolean[length][height];
		for (int i = 0; i < length; i = i + 1)
			for (int j = 0; j < height; j = j + 1)
				newArray[i][j] = checkCell(i, j);
		array = newArray;
	}
	
	// output data as text					
	public void printArray() {
		for (int j = 0; (j < height); j = j + 1) {
			for (int i = 0; (i < length); i = i + 1)
				if (array[i][j]) 
					System.out.print("O");
				else
					System.out.print(" ");
			System.out.println("");
		}
	}
	
	// create random initial grid
	public void initialData() {
		Random r = new Random();
		for (int i = 1; (i < length - 1); i = i + 1)
			for (int j = 1; (j < height - 1); j = j + 1)				
				array[i][j] = (r.nextBoolean());
	}
}











