/**********************************************************************\
 * Main
 * 		Main window (contains grid and UI options)
\**********************************************************************/

package Life;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class Main extends javax.swing.JFrame implements ActionListener {
	private static IconRenderer iconRenderer = new IconRenderer();
	private CellGrid grid;
	private JButton button; 
	private JButton playagain;
	private int selcol = -1;
	private int selrow = -1;

	// create grid with random data
	public Main(int width, int height) {
		grid = new CellGrid(width, height);
		grid.initialData();
	}

	// populate window with grid and buttons
	public void initComponents() throws Exception {
		// the following code sets the frame's initial state
		setTitle("Life.Main");

		// next generation: time step
		button = new JButton("Next Generation");
		button.setMnemonic(KeyEvent.VK_1);
		button.addActionListener(this);
		button.setActionCommand("next");
		button.setAlignmentX(Component.CENTER_ALIGNMENT);

		// play again: reinitialize grid
		playagain = new JButton("Play Again?");
		playagain.addActionListener(this);
		playagain.setActionCommand("new");
		playagain.setAlignmentX(Component.CENTER_ALIGNMENT);
	
		// table with life cells: click to toggle
		JTable table = new JTable(new MyTableModel(grid))
			{
				public TableCellRenderer getCellRenderer(int row, int column) {
					return iconRenderer;
				}
			};

		table.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				ToggleValue();
			}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
		});

		// Ask to be notified of selection changes.
		ListSelectionModel rowSM = table.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        // Ignore extra messages.
		        if (e.getValueIsAdjusting()) return;

		        ListSelectionModel lsm =
        		    (ListSelectionModel) e.getSource();
		        if (lsm.isSelectionEmpty()) {
		        	selrow = -1;
		        } else {
        		    selrow = lsm.getMinSelectionIndex();
        		}
		    }
		});

		// get column selections too
        ListSelectionModel colSM =
            table.getColumnModel().getSelectionModel();
        colSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                // Ignore extra messages.
                if (e.getValueIsAdjusting()) return;
                ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                if (lsm.isSelectionEmpty()) {
					selcol = -1;
                } else {
                    selcol = lsm.getMinSelectionIndex();
                }
            }
        });
            
		TableColumn column = null;
		for (int i = 0; i < table.getColumnCount(); i++) {
    		table.getColumnModel().getColumn(i).setPreferredWidth(12);
		}
   		table.getColumnModel().setColumnSelectionAllowed(false);
		table.setRowSelectionAllowed(false);
		table.clearSelection();

		// layout items in window
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(BorderFactory.createEmptyBorder(30, 30, 10, 30));
		panel.add(table);
		panel.add(Box.createRigidArea(new Dimension(0,5)));
		panel.add(button);
		panel.add(playagain);

		getRootPane().setDefaultButton(button);
		getContentPane().add(panel, BorderLayout.CENTER);
		pack();

		// event handling
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				thisWindowClosing(e);
			}
		});
	}

	// listen for button clicks
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "next") {
			grid.makeArray();
		} else {
			grid.initialData();
		}
		
		repaint();		
	}
	
	// called for clicks on grid
	public void ToggleValue() {
		grid.setCell(selrow, selcol, !grid.getCell(selrow, selcol));
	}

	// Close the window when the close box is clicked
	void thisWindowClosing(java.awt.event.WindowEvent e) {
		setVisible(false);
		dispose();
		System.exit(0);
	}
	
}
